@@ -1,5 +1,5 @@
 /*
- * $Id: dbus.c,v 1.10 2004/08/14 15:22:23 tal197 Exp $
+ * $Id: dbus.c,v 1.12 2005/07/03 22:21:31 tal197 Exp $
  *
  * ROX-Session, a very simple session manager
  * Copyright (C) 2002, Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -43,15 +43,18 @@
 #include "main.h"
 
 static gint dbus_pid = -1;
-static DBusConnection *dbus_connection = NULL;
+
 static void *dbus_connection_glib = NULL;	/* For D-BUS 0.22 and later */
+static DBusConnection *dbus_connection = NULL;
+
+#ifndef DBUS_03
 static DBusGProxy *(*_dbus_gproxy_new_for_peer)
 		(DBusConnection *, const char *, const char *) = NULL;
-
 /* We dlopen libdbus-1.so to avoid binary compatibility problems. */
 static void *libdbus = NULL;
 static void *libdbus_glib = NULL;
 static int dbus_version = -1;	/* The minor part of the version number (20 or 22) */
+#endif
 
 /* Read one line from fd and store in DBUS_SESSION_BUS_ADDRESS. */
 static void read_address(gint fd)
@@ -200,7 +203,9 @@ static DBusMessage *session_handler(DBusMessage *message, DBusError *error)
  */
 gboolean register_object_path(const char *path, MessageHandler handler)
 {
+#ifndef DBUS_03
 	void *reg;
+#endif
 	DBusObjectPathVTable vtable = {
 		NULL,
 		message_handler,
@@ -210,14 +215,18 @@ gboolean register_object_path(const char *path, MessageHandler handler)
 	g_return_val_if_fail(strchr(path + 1, '/') == NULL, FALSE);
 	g_return_val_if_fail(dbus_connection != NULL, FALSE);
 
+#ifdef DBUS_03
+	if (!dbus_connection_register_object_path(dbus_connection, path, &vtable, handler))
+		goto oom;
+#else
 	reg = dlsym(libdbus, "dbus_connection_register_object_path");
 	g_return_val_if_fail(reg != NULL, FALSE);
 
 	if (dbus_version == 20)
 	{
 		const char *pathv[] = {NULL, NULL};
-		gboolean (*dbus_register)(DBusConnection *, const char **,
-					  DBusObjectPathVTable *, MessageHandler);
+		dbus_bool_t (*dbus_register)(DBusConnection *, const char **,
+					  DBusObjectPathVTable *, void *);
 
 		dbus_register = reg;
 
@@ -227,14 +236,15 @@ gboolean register_object_path(const char *path, MessageHandler handler)
 	}
 	else
 	{
-		gboolean (*dbus_register)(DBusConnection *, const char *,
-					  DBusObjectPathVTable *, MessageHandler);
+		dbus_bool_t (*dbus_register)(DBusConnection *, const char *,
+					  DBusObjectPathVTable *, void *);
 
 		dbus_register = reg;
 
 		if (!dbus_register(dbus_connection, path, &vtable, handler))
 			goto oom;
 	}
+#endif
 	
 	return TRUE;
 
@@ -246,6 +256,12 @@ oom:
 /* Also sets dbus_version and _dbus_gproxy_new_for_peer */
 static DBusConnection *_dbus_wrapper_get_session_bus(GError **error)
 {
+#ifdef DBUS_03
+	g_print("Using D-BUS 0.3 interface.\n");
+	dbus_connection_glib = dbus_g_bus_get(DBUS_BUS_SESSION, error);
+	g_return_val_if_fail(dbus_connection_glib != NULL, NULL);
+	return (DBusConnection *) dbus_g_connection_get_connection(dbus_connection_glib);
+#else
 	void *(*bus_get)(int, GError **);
 	
 	bus_get = dlsym(libdbus_glib, "dbus_g_bus_get");
@@ -258,7 +274,7 @@ static DBusConnection *_dbus_wrapper_get_session_bus(GError **error)
 							"dbus_g_proxy_new_for_peer");
 		g_return_val_if_fail(_dbus_gproxy_new_for_peer != NULL, NULL);
 		
-		dbus_version = 22;	/* 0.20 or 0.21 */
+		dbus_version = 22;	/* 0.22 or later */
 		dbus_connection_glib = bus_get(DBUS_BUS_SESSION, error);
 		g_return_val_if_fail(dbus_connection_glib != NULL, NULL);
 		
@@ -276,9 +292,10 @@ static DBusConnection *_dbus_wrapper_get_session_bus(GError **error)
 
 		bus_get = dlsym(libdbus_glib, "dbus_bus_get_with_g_main");
 		g_return_val_if_fail(bus_get != NULL, NULL);
-		dbus_version = 20;
+		dbus_version = 20; 		/* 0.20 or 0.21 */
 		return bus_get(DBUS_BUS_SESSION, error);
 	}
+#endif
 }
 
 /* TRUE on success */
@@ -295,7 +312,11 @@ static gboolean connect_to_bus(void)
 		goto err;
 	dbus_connection_set_exit_on_disconnect(dbus_connection, FALSE);
 
+#ifdef DBUS_03
+	if (dbus_bus_name_has_owner(dbus_connection,
+#else
 	if (dbus_bus_service_exists(dbus_connection,
+#endif
 				test_mode ? ROX_SESSION_DBUS_SERVICE_TEST
 					  : ROX_SESSION_DBUS_SERVICE,
 				&derror))
@@ -308,7 +329,11 @@ static gboolean connect_to_bus(void)
 	if (error)
 		goto err;
 	
+#ifdef DBUS_03
+	dbus_bus_request_name(dbus_connection,
+#else
 	dbus_bus_acquire_service(dbus_connection,
+#endif
 			test_mode ? ROX_SESSION_DBUS_SERVICE_TEST
 				  : ROX_SESSION_DBUS_SERVICE,
 			0,
@@ -320,10 +345,16 @@ static gboolean connect_to_bus(void)
 	register_object_path("/Session", session_handler);
 
 	/* Get notified when the bus dies */
+#ifdef DBUS_03
+	local = dbus_g_proxy_new_for_peer(
+			dbus_connection_glib,
+			DBUS_PATH_LOCAL, DBUS_INTERFACE_LOCAL);
+#else
 	local = _dbus_gproxy_new_for_peer(
 			dbus_connection_glib ? dbus_connection_glib : dbus_connection,
 			DBUS_PATH_ORG_FREEDESKTOP_LOCAL,
 			DBUS_INTERFACE_ORG_FREEDESKTOP_LOCAL);
+#endif
 	g_return_val_if_fail(local != NULL, FALSE);
 	g_signal_connect(G_OBJECT(local), "destroy",
 			G_CALLBACK(lost_connection), NULL);
@@ -354,8 +385,13 @@ void dbus_init(void)
 {
 	GError *error = NULL;
 	gint stdout_pipe = -1;
+#ifdef DBUS_03
+	gchar *argv[] = {"dbus-daemon", "--session", "--print-address", NULL};
+#else
 	gchar *argv[] = {"dbus-daemon-1", "--session", "--print-address", NULL};
+#endif
 
+#ifndef DBUS_03
 	libdbus = dlopen("libdbus-1.so.0", RTLD_LAZY | RTLD_NOLOAD);
 	if (!libdbus)
 	{
@@ -370,6 +406,7 @@ void dbus_init(void)
 			    "D-BUS support disabled."));
 		return;
 	}
+#endif
 
 	if (getenv("DBUS_SESSION_BUS_ADDRESS") != NULL)
 	{
