@@ -1,5 +1,5 @@
 /*
- * $Id: dbus.h,v 1.3 2004/08/14 15:22:23 tal197 Exp $
+ * $Id: dbus.h,v 1.4 2005/07/03 22:21:31 tal197 Exp $
  *
  * ROX-Session, a very simple session manager
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -12,6 +12,10 @@
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 
+#ifdef DBUS_03
+#include <dbus/dbus-glib-lowlevel.h>
+#endif
+
 #define ROX_SESSION_ERROR "net.sf.rox.Session.Error"
 
 typedef DBusMessage *(*MessageHandler)(DBusMessage *message, DBusError *error);
