@@ -1,5 +1,5 @@
 /* vi: set cindent:
- * $Id: log.c,v 1.21 2004/07/10 14:25:18 tal197 Exp $
+ * $Id: log.c,v 1.22 2005/05/12 19:36:43 tal197 Exp $
  *
  * ROX-Session, a very simple session manager
  * Copyright (C) 2002, Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -403,7 +403,18 @@ static void got_log_data(gpointer data,
 	}
 	else
 	{
-		log_msg(buffer, got);
+		if (g_utf8_validate(buffer, got, NULL)) {
+			log_msg(buffer, got);
+		} else {
+			char *fixed;
+
+			fixed = g_convert_with_fallback(buffer, got,
+					"utf-8", "iso-8859-1",
+					"?", NULL, NULL, NULL);
+			log_msg(fixed, -1);
+			log_msg("(invalid UTF-8)\n", -1);
+			g_free(fixed);
+		}
 
 		if (real_stderr != -1)
 			write_stderr(buffer, got);
